package RSSReader::Gtk2UI;
use Gtk2;
use Gtk2::Pango;
use Gnome2;
use strict;
use Data::Dumper;
use RSSReader::Gtk2UI::MainWindow;
use RSSReader::GtkUI::Preferences;
use RSSReader::Configuration;
use RSSReader;
use RSSReader::Gtk2UI::Dialog;
use RSSReader::Gtk2UI::Fetcher;
use base qw(RSSReader::Logger RSSReader::Notifier);

#set_locale Gtk;
#init Gtk;
Gnome2::Program->init ('Syndigator', '1.0beta', 'libgnomeui',
                       show_crash_dialog => 0,
		       app_libdir => './');

our $_gtk_main_info = ();

sub new {
	my $c = shift;
	return $_gtk_main_info if $_gtk_main_info;

	$_gtk_main_info = bless { }, $c;

	return $_gtk_main_info;
}

sub launch_url {
	my $s   = shift;
	my $url = shift;
	Gnome2::HRef->new($url,$url)->clicked;
	return 1;
}

sub run {
	my $c = shift;
	my %a = @_;
	my $reader = RSSReader->new();
	
	my $prefs = RSSReader::GtkUI::Preferences->new;
	$prefs->load;
	RSSReader::Configuration->new();

	if ($a{offline}) {
           $c->log_debug("Starting offline");
           $prefs->online(0);
        }

	my $reader = RSSReader->new();
	RSSReader::Gtk2UI::MainWindow->new()->show();
        $prefs->notify_me(event=>'auto_save_changed',          caller=>'RSSReader::Gtk2UI', call=>'_set_up_auto_save');
        $prefs->notify_me(event=>'auto_save_interval_changed', caller=>'RSSReader::Gtk2UI', call=>'_set_up_auto_save');
        $prefs->notify_me(event=>'online_changed'           ,  caller=>'RSSReader::Gtk2UI', call=>'_set_up_fetch');
        $c->_set_up_auto_save();
        $c->_set_up_fetch();
	
	$c->get_feeds;

	main Gtk2;
	exit( 0 );	
}

sub _set_up_auto_save {
        my $prefs = RSSReader::GtkUI::Preferences->new();
                                                                                                                                                                   
        if ($_gtk_main_info->{"_auto_save_timeout"}) {
          __PACKAGE__->log_debug("Removing previous autosave timeout");
          Glib::Source->remove($_gtk_main_info->{"_auto_save_timeout"});
          delete $_gtk_main_info->{"_auto_save_timeout"};
        }
                                                                                                                                                                   
        if ( $prefs->auto_save() ) {
          __PACKAGE__->log_debug("Setting up auto save (".$prefs->auto_save().")");
          $_gtk_main_info->{"_auto_save_timeout"} = Glib::Timeout->add( $prefs->auto_save_interval*60*1000, sub { _auto_save()  } );
        }
}

sub _set_up_fetch {
	my $c = shift;
        my $prefs = RSSReader::GtkUI::Preferences->new();
        if ($_gtk_main_info->{_refetch_time}) {
                __PACKAGE__->log_debug("Removing previous auto fetch");
                Glib::Source->remove( $_gtk_main_info->{_refetch_time} );
                delete $_gtk_main_info->{_refetch_time}
        }
        if ( $prefs->online() ) {
                $_gtk_main_info->{_refetch_time} = Glib::Timeout->add( 5*60*1000, sub { RSSReader::Gtk2UI->get_feeds() } );
        }

}


sub _auto_save {
       #FIXME
       #SHOULD run a wizzard...
       my $mainwindow = RSSReader::Gtk2UI::MainWindow->new();
       my $mid = $mainwindow->status_bar_add( 'Autosaving ' );
       __PACKAGE__->log_debug("Auto saving");
       RSSReader::Configuration->new()->save();
       RSSReader::GtkUI::Preferences->new()->save();
       $mainwindow->status_bar_remove($mid);
       return 1;
}

sub get_feeds {
        my $c = shift;
        my %a = @_;
	$c->log_debug("They want me to get feeds");
        return unless RSSReader::GtkUI::Preferences->new()->online();
        my @to_fetch = ();
        foreach my $feed (@{RSSReader::Feed->get_all_feeds()}) {
                my $this_feed_f_interval = $feed->fetch_interval || RSSReader::GtkUI::Preferences->new()->fetch_interval;
                next unless ( (($feed->last_fetched + $this_feed_f_interval*60) < time()) || $a{force});
                $c->log_debug("Should fetch ".$feed->label()." ($this_feed_f_interval) (".$feed->last_fetched().")");
                push @to_fetch, $feed;
        }
        foreach my $feed (@to_fetch) { $c->get_feed($feed) }
        return 1;
}

sub get_feed {
        my $c = shift;
        return unless RSSReader::GtkUI::Preferences->new()->online();
        my $feed = shift;
        my $mainwindow = RSSReader::Gtk2UI::MainWindow->new();
	$c->log_debug("Fetching feed ".$feed->label);
        my $mid = $mainwindow->status_bar_add( 'Fetching '.$feed->label() );
        Gtk2->main_iteration while ( Gtk2->events_pending );
        RSSReader::Gtk2UI::Fetcher->fetch_feed($feed);
        Gtk2->main_iteration while ( Gtk2->events_pending );
        $mainwindow->status_bar_remove($mid);
}


sub _exit {
  my $s = shift;
  $s->log_debug("GUI exiting");
  $s->notify('exit');
  my $conf  = RSSReader::Configuration->new();
  my $prefs = RSSReader::GtkUI::Preferences->new();
  if (!$prefs) {
        RSSReader::Gtk2UI::Dialog->run($@);
        return;
  }

  $conf->save()  if $prefs->save_on_exit;
  $prefs->save() if $prefs->save_on_exit;

  Gtk2->main_quit;
}

sub _port_configuration {
}

1;
